@@ -43,6 +43,8 @@ SDLAppDisplay::SDLAppDisplay() {
     vsync           = false;
     resizable       = false;
     frameless       = false;
+    experimental    = false;
+    high_dpi_aware  = false;
     multi_sample    = 0;
     width           = 0;
     height          = 0;
@@ -50,9 +52,15 @@ SDLAppDisplay::SDLAppDisplay() {
     desktop_height  = 0;
     windowed_width  = 0;
     windowed_height = 0;
+    viewport_dpi_ratio = vec2(1.0f, 1.0f);
 #if SDL_VERSION_ATLEAST(2,0,0)
     sdl_window = 0;
     gl_context = 0;
+
+    framed_width  = 0;
+    framed_height = 0;
+    framed_x      = 0;
+    framed_y      = 0;
 #else
     surface = 0;
 #endif
@@ -75,8 +83,12 @@ Uint32 SDLAppDisplay::SDLWindowFlags(bool fullscreen) {
     Uint32 flags = SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN;
 
     if (frameless) flags |= SDL_WINDOW_BORDERLESS;
-    if (resizable) flags |= SDL_WINDOW_RESIZABLE;
+    if (resizable && !frameless) flags |= SDL_WINDOW_RESIZABLE;
     if (fullscreen) flags |= SDL_WINDOW_FULLSCREEN;
+#ifndef _WIN32
+    // see setWindowsHighDPIAwareness() work around for Windows
+    if(high_dpi_aware) flags |= SDL_WINDOW_ALLOW_HIGHDPI;
+#endif
 #else
     Uint32 flags = SDL_OPENGL | SDL_HWSURFACE | SDL_ANYFORMAT | SDL_DOUBLEBUF;
 
@@ -99,19 +111,38 @@ void SDLAppDisplay::enableResize(bool resizable) {
     this->resizable = resizable;
 }
 
+void SDLAppDisplay::enableFrameless(bool frameless) {
+    this->frameless = frameless;
+}
+
+void SDLAppDisplay::enableHighDPIAwareness(bool enable) {
+    high_dpi_aware = enable;
+}
+
 void SDLAppDisplay::enableAlpha(bool enable) {
     enable_alpha = enable;
 }
 
+void SDLAppDisplay::enableExperimental(bool enable) {
+    experimental = enable;
+}
+
 void SDLAppDisplay::multiSample(int samples) {
     multi_sample = samples;
 }
 
 void SDLAppDisplay::setupExtensions() {
 
+    if(experimental) glewExperimental = true;
+
     GLenum err = glewInit();
 
+#ifdef GLEW_ERROR_NO_GLX_DISPLAY
+    // Ignore GLEW_ERROR_NO_GLX_DISPLAY error as wont have GLX if using Wayland
+    if (GLEW_OK != err && GLEW_ERROR_NO_GLX_DISPLAY != err) {
+#else
     if (GLEW_OK != err) {
+#endif
         /* Problem: glewInit failed, something is seriously wrong. */
         char glewerr[1024];
         snprintf(glewerr, 1024, "GLEW Error: %s", glewGetErrorString(err));
@@ -139,7 +170,18 @@ LRESULT CALLBACK window_filter_proc(HWND wnd, UINT msg, WPARAM wparam, LPARAM lp
 }
 #endif
 
-void SDLAppDisplay::setVideoMode(int width, int height, bool fullscreen) {
+void SDLAppDisplay::updateViewportDPIRatio() {
+    int drawable_width, drawable_height;
+    SDL_GL_GetDrawableSize(sdl_window, &drawable_width, &drawable_height);
+    
+    int sdl_window_width, sdl_window_height;
+    SDL_GetWindowSize(sdl_window, &sdl_window_width, &sdl_window_height);
+
+    viewport_dpi_ratio = vec2(drawable_width, drawable_height) / vec2(sdl_window_width, sdl_window_height);
+    debugLog("viewport dpi ratio %.2f x %.2f", viewport_dpi_ratio.x, viewport_dpi_ratio.y);
+}
+
+void SDLAppDisplay::setVideoMode(int width, int height, bool fullscreen, int screen) {
 #if SDL_VERSION_ATLEAST(2,0,0)
 
     SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
@@ -149,17 +191,40 @@ void SDLAppDisplay::setVideoMode(int width, int height, bool fullscreen) {
         SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
         SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, (GLuint) multi_sample);
     }
-    
+
     if(enable_alpha) {
         SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
     }
-    
+
     Uint32 flags = SDLWindowFlags(fullscreen);
 
     if(gl_context != 0) SDL_GL_DeleteContext(gl_context);
-    if(sdl_window != 0) SDL_DestroyWindow(sdl_window);
 
-    sdl_window = SDL_CreateWindow(gSDLAppTitle.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, width, height, flags);
+
+    int position_x = -1;
+    int position_y = -1;
+
+    int display_index = -1;
+
+    if(sdl_window != 0) {
+        display_index = SDL_GetWindowDisplayIndex(sdl_window);
+        SDL_GetWindowPosition(sdl_window, &position_x, &position_y);
+        SDL_DestroyWindow(sdl_window);
+
+    } else if(screen > 0 && screen <= SDL_GetNumVideoDisplays()) {
+        display_index = screen-1;
+    }
+
+    if(display_index != -1) {
+        sdl_window = SDL_CreateWindow(gSDLAppTitle.c_str(), SDL_WINDOWPOS_UNDEFINED_DISPLAY(display_index), SDL_WINDOWPOS_UNDEFINED_DISPLAY(display_index), width, height, flags);
+
+        if(sdl_window && position_x >= 0 && position_y >= 0) {
+            SDL_SetWindowPosition(sdl_window, position_x, position_y);
+        }
+
+    } else {
+        sdl_window = SDL_CreateWindow(gSDLAppTitle.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, width, height, flags);
+    }
 
     if (!sdl_window) {
 
@@ -169,7 +234,15 @@ void SDLAppDisplay::setVideoMode(int width, int height, bool fullscreen) {
             SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 0);
             SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 0);
 
-            sdl_window = SDL_CreateWindow(gSDLAppTitle.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, width, height, flags);
+            if(display_index != -1) {
+                sdl_window = SDL_CreateWindow(gSDLAppTitle.c_str(), SDL_WINDOWPOS_UNDEFINED_DISPLAY(display_index), SDL_WINDOWPOS_UNDEFINED_DISPLAY(display_index), width, height, flags);
+
+                if(sdl_window && position_x >= 0 && position_y >= 0) {
+                    SDL_SetWindowPosition(sdl_window, position_x, position_y);
+                }
+            } else {
+                sdl_window = SDL_CreateWindow(gSDLAppTitle.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, width, height, flags);
+            }
         }
 
         if(!sdl_window) {
@@ -302,7 +375,10 @@ void SDLAppDisplay::toggleFullscreen() {
 
 #if SDL_VERSION_ATLEAST(2,0,0)
     SDL_SetWindowFullscreen(sdl_window, fullscreen ? SDL_WINDOW_FULLSCREEN_DESKTOP : 0);
-    SDL_GetWindowSize(sdl_window, &resized_width, &resized_height);
+
+    SDL_GL_GetDrawableSize(sdl_window, &resized_width, &resized_height);
+
+    updateViewportDPIRatio();
 #else
     setVideoMode(width, height, fullscreen);
 
@@ -319,12 +395,137 @@ void SDLAppDisplay::toggleFullscreen() {
     this->height = resized_height;
 }
 
+void SDLAppDisplay::toggleFrameless() {
+#ifdef __APPLE__
+    debugLog("Frameless toggle not supported");
+    return;
+#endif
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+    if(fullscreen) return;
+
+    frameless = !frameless;
+
+    if(frameless) {
+
+        int position_x, position_y;
+        SDL_GetWindowPosition(sdl_window, &position_x, &position_y);
+
+        framed_width  = width;
+        framed_height = height;
+        framed_x      = position_x;
+        framed_y      = position_y;
+
+#ifdef _WIN32
+        SDL_SysWMinfo sys_window_info;
+        SDL_VERSION(&sys_window_info.version);
+
+        if(SDL_GetWindowWMInfo(sdl_window, &sys_window_info)) {
+
+            //make the new window equal the size of the old window including frame
+
+            HWND wnd = sys_window_info.info.win.window;
+
+            RECT rect;
+            GetWindowRect(wnd, &rect);
+
+            position_x = rect.left;
+            position_y = rect.top;
+
+            width  = rect.right - rect.left;
+            height = rect.bottom - rect.top;
+        }
+#endif
+
+        //work around window position changing when when frame is toggled
+        //related bug: https://bugzilla.libsdl.org/show_bug.cgi?id=2791
+
+        SDL_SetWindowBordered(sdl_window, SDL_FALSE);
+        SDL_SetWindowSize(sdl_window, width, height);
+        SDL_SetWindowPosition(sdl_window, position_x, position_y);
+
+        //window needs to be recreated to remove SDL_WINDOW_RESIZABLE flag
+        //otherwise there is still a weird border
+
+        setVideoMode(width, height, fullscreen);
+
+    } else {
+
+#ifdef _WIN32
+        // handle computing framed window position
+        // if launched in frameless mode initially
+        if(framed_width == 0) {
+            SDL_SysWMinfo sys_window_info;
+            SDL_VERSION(&sys_window_info.version);
+
+            if(SDL_GetWindowWMInfo(sdl_window, &sys_window_info)) {
+
+                HWND wnd = sys_window_info.info.win.window;
+
+                RECT old_rect;
+                GetWindowRect(wnd, &old_rect);
+
+                SDL_SetWindowBordered(sdl_window, SDL_TRUE);
+
+                RECT new_rect;
+                GetWindowRect(wnd, &new_rect);
+
+                SDL_GetWindowSize(sdl_window, &framed_width, &framed_height);
+                SDL_GetWindowPosition(sdl_window, &framed_x, &framed_y);
+
+                int width_delta  = (new_rect.right - new_rect.left) - (old_rect.right - old_rect.left);
+                int height_delta = (new_rect.bottom - new_rect.top) - (old_rect.bottom - old_rect.top);
+
+                framed_width  = width - width_delta;
+                framed_height = height - height_delta;
+
+                framed_x += width_delta;
+                framed_y += height_delta;
+
+                // HACK: account for the resizable windows border being 2 pixels wider
+
+                if(resizable) {
+                    framed_x += 2;
+                    framed_y += 2;
+                }
+            }
+        }
+#endif
+        SDL_SetWindowBordered(sdl_window, SDL_TRUE);
+
+        if(framed_width > 0) {
+            width  = framed_width;
+            height = framed_height;
+        }
+
+        SDL_SetWindowSize(sdl_window, width, height);
+
+        if(framed_width > 0) {
+            SDL_SetWindowPosition(sdl_window, framed_x, framed_y);
+        }
+
+        setVideoMode(width, height, fullscreen);
+    }
+#endif
+}
+
+bool SDLAppDisplay::isFullscreen() const {
+    return fullscreen;
+}
+
+bool SDLAppDisplay::isFrameless() const {
+    return frameless;
+}
+
 void SDLAppDisplay::resize(int width, int height) {
+    debugLog("resize %d x %d", width, height);
 
     int resized_width, resized_height;
 
 #if SDL_VERSION_ATLEAST(2,0,0)
-    SDL_GetWindowSize(sdl_window, &resized_width, &resized_height);
+    SDL_GL_GetDrawableSize(sdl_window, &resized_width, &resized_height);
+
+    updateViewportDPIRatio();
 #else
     setVideoMode(width, height, fullscreen);
 
@@ -341,18 +542,64 @@ void SDLAppDisplay::resize(int width, int height) {
     this->height = resized_height;
 }
 
-void SDLAppDisplay::init(std::string window_title, int width, int height, bool fullscreen) {
+
+#ifdef _WIN32
+void setWindowsHighDPIAwareness() {
+    // Because SDL2 doesn't currently do anything if you set SDL_WINDOW_ALLOW_HIGHDPI
+    // on Windows, it is necessary to manually tell Windows your application supports high DPI,
+    // otherwise it will give you a 1920x1080 window when you request 3840x2160 on a 4K display
+
+    // https://discourse.libsdl.org/t/sdl-getdesktopdisplaymode-resolution-reported-in-windows-10-when-using-app-scaling/22389/3
+
+    typedef enum PROCESS_DPI_AWARENESS {
+      PROCESS_DPI_UNAWARE = 0,
+      PROCESS_SYSTEM_DPI_AWARE = 1,
+      PROCESS_PER_MONITOR_DPI_AWARE = 2
+    } ;
+
+    BOOL(WINAPI *SetProcessDPIAware)(void) = NULL;
+    HRESULT(WINAPI *SetProcessDpiAwareness)(PROCESS_DPI_AWARENESS dpiAwareness) = NULL;
+
+    if (void* userDLL = SDL_LoadObject("USER32.DLL")) {
+        SetProcessDPIAware = (BOOL(WINAPI *)(void)) SDL_LoadFunction(userDLL, "SetProcessDPIAware");
+    }
+
+    if (void* shcoreDLL = SDL_LoadObject("SHCORE.DLL")) {
+        SetProcessDpiAwareness = (HRESULT(WINAPI *)(PROCESS_DPI_AWARENESS)) SDL_LoadFunction(shcoreDLL, "SetProcessDpiAwareness");
+    }
+
+    if (SetProcessDpiAwareness) {
+        // Windows 8.1 and later
+        HRESULT result = SetProcessDpiAwareness(PROCESS_PER_MONITOR_DPI_AWARE);
+        debugLog("SetProcessDpiAwareness %d", (result == S_OK) ? 1 : 0);
+
+    } else if (SetProcessDPIAware) {
+        // Vista / Windows 8
+        BOOL success = SetProcessDPIAware();
+        debugLog("SetProcessDPIAware %d", (int)success);
+    }
+}
+#endif
+
+void SDLAppDisplay::init(std::string window_title, int width, int height, bool fullscreen, int screen) {
 
     if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO) != 0) {
         throw SDLInitException(SDL_GetError());
     }
 
+#ifdef _WIN32
+    if(high_dpi_aware) setWindowsHighDPIAwareness();
+#endif
 
 #if SDL_VERSION_ATLEAST(2,0,0)
 
-    // TODO: which display? is 0 the designated primary display always?
+    // check screen is valid
+    if(screen <= 0 || screen > SDL_GetNumVideoDisplays()) {
+        screen = -1;
+    }
+
     SDL_Rect display_rect;
-    SDL_GetDisplayBounds(0, &display_rect);
+    SDL_GetDisplayBounds(screen > 0 ? screen-1 : 0, &display_rect);
 
     desktop_width  = display_rect.w;
     desktop_height = display_rect.h;
@@ -384,7 +631,9 @@ void SDLAppDisplay::init(std::string window_title, int width, int height, bool f
     SDL_WM_SetCaption(window_title.c_str(),0);
 #endif
 
-    setVideoMode(width, height, fullscreen);
+    debugLog("requesting video mode %d x %d", width, height);
+
+    setVideoMode(width, height, fullscreen, screen);
 
     //get actual opengl viewport
     GLint viewport[4];
@@ -394,6 +643,15 @@ void SDLAppDisplay::init(std::string window_title, int width, int height, bool f
     this->height     = viewport[3];
     this->fullscreen = fullscreen;
 
+    debugLog("opengl viewport %d x %d", this->width, this->height);
+
+    int drawable_width, drawable_height;
+    SDL_GL_GetDrawableSize(this->sdl_window, &drawable_width, &drawable_height);
+
+    debugLog("drawable size %d x %d", drawable_width, drawable_height);
+
+    updateViewportDPIRatio();
+
     glViewport(0, 0, this->width, this->height);
 }
 
