@@ -26,6 +26,7 @@
 */
 
 #include "resource.h"
+#include <boost/filesystem.hpp>
 
 ResourceManager::ResourceManager() {
 }
@@ -50,17 +51,12 @@ void ResourceManager::purge() {
 ResourceManager::~ResourceManager() {
 }
 
-Resource* ResourceManager::grab(const std::string& name) {
-    //debugLog("grabing %s\n", name.c_str());
-    Resource* r = resources[name];
+bool ResourceManager::fileExists(const std::string& filename) {
+    return boost::filesystem::is_regular_file(filename);
+}
 
-    if(r==0) {
-        //debugLog("%s not found. creating resource...\n", name.c_str());
-        r = create(name);
-        resources[name] = r;
-    }
-    r->addref();
-    return r;
+bool ResourceManager::dirExists(const std::string& dirname) {
+    return boost::filesystem::is_directory(dirname);
 }
 
 void ResourceManager::release(Resource* resource) {
@@ -77,4 +73,3 @@ void ResourceManager::release(Resource* resource) {
         delete r;
     }
 }
-
