@@ -36,11 +36,22 @@ Gource::Gource(FrameExporter* exporter) {
         gGourceSettings.file_graphic = texturemanager.grab("file.png", true, GL_CLAMP_TO_EDGE);
     }
 
-    fontlarge = fontmanager.grab(gGourceSettings.font_file, 42);
+    if(gGourceSettings.default_font_scale) {
+        if(display.viewport_dpi_ratio.x > 1.0f) {
+            gGourceSettings.font_scale = display.viewport_dpi_ratio.x;
+        } else {
+            int threshold = 1600;
+            gGourceSettings.font_scale = (float) (1 + glm::max(0, display.height / threshold));
+        }
+        debugLog("setting font scale for resolution %d x %d to %.2f", display.width, display.height, gGourceSettings.font_scale);
+        gGourceSettings.setScaledFontSizes();
+    }
+
+    fontlarge = fontmanager.grab(gGourceSettings.font_file, 42 * gGourceSettings.font_scale);
     fontlarge.dropShadow(true);
     fontlarge.roundCoordinates(true);
 
-    fontmedium = fontmanager.grab(gGourceSettings.font_file, gGourceSettings.font_size);
+    fontmedium = fontmanager.grab(gGourceSettings.font_file, gGourceSettings.scaled_font_size);
     fontmedium.dropShadow(true);
     fontmedium.roundCoordinates(false);
 
@@ -49,14 +60,16 @@ Gource::Gource(FrameExporter* exporter) {
     fontcaption.roundCoordinates(false);
     fontcaption.alignTop(false);
 
-    font = fontmanager.grab(gGourceSettings.font_file, 14);
+    font = fontmanager.grab(gGourceSettings.font_file, 14 * gGourceSettings.font_scale);
     font.dropShadow(true);
     font.roundCoordinates(true);
 
-    fontdirname = fontmanager.grab(gGourceSettings.font_file, gGourceSettings.dirname_font_size);
+    fontdirname = fontmanager.grab(gGourceSettings.font_file, gGourceSettings.scaled_dirname_font_size);
     fontdirname.dropShadow(true);
     fontdirname.roundCoordinates(true);
 
+    slider.init();
+
     //only use bloom with alpha channel if transparent due to artifacts on some video cards
     std::string bloom_tga = gGourceSettings.transparent ? "bloom_alpha.tga" : "bloom.tga";
 
@@ -124,14 +137,15 @@ Gource::Gource(FrameExporter* exporter) {
     hoverUser = 0;
 
     date_x_offset = 0;
+    starting_z = -300.0f;
 
-    textbox = TextBox(fontmanager.grab(gGourceSettings.font_file, 18));
+    textbox = TextBox(fontmanager.grab(gGourceSettings.font_file, 18 * gGourceSettings.font_scale));
     textbox.setBrightness(0.5f);
     textbox.show();
 
     file_key = FileKey(1.0f);
 
-    camera = ZoomCamera(vec3(0,0, -300), vec3(0.0, 0.0, 0.0), gGourceSettings.camera_zoom_default, gGourceSettings.camera_zoom_max);
+    camera = ZoomCamera(vec3(0,0, starting_z), vec3(0.0, 0.0, 0.0), gGourceSettings.camera_zoom_default, gGourceSettings.camera_zoom_max);
     camera.setPadding(gGourceSettings.padding);
 
     setCameraMode(gGourceSettings.camera_mode);
@@ -326,6 +340,8 @@ void Gource::mouseMove(SDL_MouseMotionEvent *e) {
     if(gGourceSettings.disable_input) return;
     if(gGourceSettings.hide_mouse) return;
 
+    // debugLog("mouse move %d, %d (change %d, %d)", e->x, e->y, e->xrel, e->yrel);
+
     if(grab_mouse) {
 #if not SDL_VERSION_ATLEAST(2,0,0)
          int warp_x = display.width/2;
@@ -1061,7 +1077,7 @@ void Gource::seekTo(float percent) {
     commitlog->seekTo(percent);
 }
 
-Regex caption_regex("^(?:\\xEF\\xBB\\xBF)?([0-9]+)\\|(.+)$");
+Regex caption_regex("^(?:\\xEF\\xBB\\xBF)?([^|]+)\\|(.+)$");
 
 void Gource::loadCaptions() {
     if(!gGourceSettings.caption_file.size()) return;
@@ -1081,7 +1097,18 @@ void Gource::loadCaptions() {
 
         if(!caption_regex.match(line, &matches)) continue;
 
-        time_t timestamp    = atol(matches[0].c_str());
+        time_t timestamp;
+
+        // Allow timestamp to be a string
+        if(matches[0].size() > 1 && matches[0].find("-", 1) != std::string::npos) {
+            if(!SDLAppSettings::parseDateTime(matches[0], timestamp))
+                continue;
+        } else {
+            timestamp = (time_t) atoll(matches[0].c_str());
+            if(!timestamp && matches[0] != "0")
+                continue;
+        }
+
         std::string caption = RCommitLog::filter_utf8(matches[1]);
 
         //ignore older captions
@@ -1150,6 +1177,10 @@ void Gource::readLog() {
         stop_position_reached = true;
     }
 
+    if((is_finished || stop_position_reached) && gGourceSettings.file_idle_time_at_end > 0.0f) {
+      gGourceSettings.file_idle_time = gGourceSettings.file_idle_time_at_end;
+    }
+
     // useful to figure out where we have crashes
     //debugLog("current date: %s\n", displaydate.c_str());
 }
@@ -2237,7 +2268,11 @@ void Gource::updateVBOs(float dt) {
             float alpha = user->getAlpha();
             vec3 col   = user->getColour();
 
-            user_vbo.add(user->graphic->textureid, user->getPos() - user->dims*0.5f, user->dims, vec4(col.x, col.y, col.z, alpha));
+            vec2 scaled_dims = user->dims;
+
+            if(gGourceSettings.fixed_user_size) scaled_dims *= (-camera.getPos().z / -starting_z);
+
+            user_vbo.add(user->graphic->textureid, user->getPos() - scaled_dims*0.5f, scaled_dims, vec4(col.x, col.y, col.z, alpha));
 
             //draw actions
             user->updateActionsVBO(action_vbo);
@@ -2581,11 +2616,11 @@ void Gource::draw(float t, float dt) {
 
     if(splash>0.0f) {
         int logowidth = fontlarge.getWidth("Gource");
-        int logoheight = 100;
+        int logoheight = 100 * gGourceSettings.font_scale;
         int cwidth    = font.getWidth("Software Version Control Visualization");
         int awidth    = font.getWidth("(C) 2009 Andrew Caudwell");
 
-        vec2 corner(display.width/2 - logowidth/2 - 30.0f, display.height/2 - 40);
+        vec2 corner(display.width/2 - logowidth/2 - 30.0f * gGourceSettings.font_scale, display.height/2 - 40 * gGourceSettings.font_scale);
 
         glDisable(GL_TEXTURE_2D);
         glColor4f(0.0f, 0.5f, 1.0f, splash * 0.015f);
@@ -2599,11 +2634,11 @@ void Gource::draw(float t, float dt) {
         glEnable(GL_TEXTURE_2D);
 
         fontlarge.setColour(vec4(1.0f));
-        fontlarge.draw(display.width/2 - logowidth/2,display.height/2 - 30, "Gource");
+        fontlarge.draw(display.width/2 - logowidth/2,display.height/2 - 30 * gGourceSettings.font_scale, "Gource");
 
         font.setColour(vec4(1.0f));
-        font.draw(display.width/2 - cwidth/2,display.height/2 + 10, "Software Version Control Visualization");
-        font.draw(display.width/2 - awidth/2,display.height/2 + 30, "(C) 2009 Andrew Caudwell");
+        font.draw(display.width/2 - cwidth/2,display.height/2 + 10 * gGourceSettings.font_scale, "Software Version Control Visualization");
+        font.draw(display.width/2 - awidth/2,display.height/2 + 30 * gGourceSettings.font_scale, "(C) 2009 Andrew Caudwell");
     }
 
     // text using the specified font goes here
