@@ -43,7 +43,7 @@ void GourceSettings::help(bool extended_help) {
 
 #ifdef _WIN32
     //resize window to fit help message
-    SDLApp::resizeConsole(1000);
+    SDLApp::resizeConsole(1040);
     SDLApp::showConsole(true);
 #endif
 
@@ -56,6 +56,9 @@ void GourceSettings::help(bool extended_help) {
     printf("  -f, --fullscreen                 Fullscreen\n");
     printf("      --screen SCREEN              Screen number\n");
     printf("      --multi-sampling             Enable multi-sampling\n");
+#ifndef _WIN32
+    printf("      --high-dpi                   Request a high DPI display\n");
+#endif
     printf("      --no-vsync                   Disable vsync\n\n");
 
     printf("  --start-date 'YYYY-MM-DD hh:mm:ss +tz'  Start at a date and optional time\n");
@@ -81,9 +84,11 @@ void GourceSettings::help(bool extended_help) {
 
     printf("  --user-image-dir DIRECTORY       Dir containing images to use as avatars\n");
     printf("  --default-user-image IMAGE       Default user image file\n");
+    printf("  --fixed-user-size                Use a fixed size throughout\n");
     printf("  --colour-images                  Colourize user images\n\n");
 
-    printf("  -i, --file-idle-time SECONDS     Time files remain idle (default: 0)\n\n");
+    printf("  -i, --file-idle-time SECONDS     Time files remain idle (default: 0)\n");
+    printf("  --file-idle-time-at-end SECONDS  Time files remain idle at end (default: 0)\n\n");
 
     printf("  --max-files NUMBER      Max number of files or 0 for no limit\n");
     printf("  --max-file-lag SECONDS  Max time files of a commit can take to appear\n\n");
@@ -268,6 +273,7 @@ GourceSettings::GourceSettings() {
     arg_types["highlight-dirs"]          = "bool";
     arg_types["file-extensions"]         = "bool";
     arg_types["file-extension-fallback"] = "bool";
+    arg_types["fixed-user-size"]         = "bool";
     arg_types["key"]                     = "bool";
     arg_types["ffp"]                     = "bool";
 
@@ -322,6 +328,7 @@ GourceSettings::GourceSettings() {
     arg_types["log-command"]        = "string";
     arg_types["background-colour"]  = "string";
     arg_types["file-idle-time"]     = "string";
+    arg_types["file-idle-time-at-end"] = "string";
     arg_types["user-image-dir"]     = "string";
     arg_types["default-user-image"] = "string";
     arg_types["date-format"]        = "string";
@@ -380,13 +387,14 @@ void GourceSettings::setGourceDefaults() {
     stop_timestamp = 0;
     stop_date = "";
 
-    start_position = 0.0f;
-    stop_position  = 0.0f;
-    stop_at_time   = -1.0f;
-    stop_on_idle   = false;
-    stop_at_end    = false;
-    dont_stop      = false;
-    no_time_travel = false;
+    start_position  = 0.0f;
+    stop_position   = 0.0f;
+    stop_at_time    = -1.0f;
+    stop_on_idle    = false;
+    stop_at_end     = false;
+    dont_stop       = false;
+    no_time_travel  = false;
+    fixed_user_size = false;
 
     show_key = false;
 
@@ -394,10 +402,11 @@ void GourceSettings::setGourceDefaults() {
 
     disable_input = false;
 
-    auto_skip_seconds = 3.0f;
-    days_per_second   = 0.1f; // TODO: check this is right
-    file_idle_time    = 0.0f;
-    time_scale        = 1.0f;
+    auto_skip_seconds     = 3.0f;
+    days_per_second       = 0.1f; // TODO: check this is right
+    file_idle_time        = 0.0f;
+    file_idle_time_at_end = 0.0f;
+    time_scale            = 1.0f;
 
     loop = false;
     loop_delay_seconds = 3.0f;
@@ -428,11 +437,14 @@ void GourceSettings::setGourceDefaults() {
 
     title             = "";
 
+    font_scale = 1.0f;
+    default_font_scale = true;
     font_file = GOURCE_FONT_FILE;
     font_size = 16;
     filename_font_size = 14;
     dirname_font_size = 14;
     user_font_size = 14;
+
     dir_colour       = vec3(1.0f);
     font_colour      = vec3(1.0f);
     highlight_colour = vec3(1.0f);
@@ -498,6 +510,16 @@ void GourceSettings::setGourceDefaults() {
         delete (*it);
     }
     user_show_filters.clear();
+
+
+    setScaledFontSizes();
+}
+
+void GourceSettings::setScaledFontSizes() {
+    scaled_font_size           = glm::clamp((int)(font_size * font_scale), 1, 100);
+    scaled_user_font_size      = glm::clamp((int)(user_font_size * font_scale), 1, 100);
+    scaled_dirname_font_size   = glm::clamp((int)(dirname_font_size * font_scale), 1, 100);
+    scaled_filename_font_size  = glm::clamp((int)(filename_font_size * font_scale), 1, 100);
 }
 
 void GourceSettings::commandLineOption(const std::string& name, const std::string& value) {
@@ -1023,16 +1045,15 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify font scale");
 
-        float font_scale = entry->getFloat();
+        font_scale = entry->getFloat();
+        default_font_scale = false;
 
         if(font_scale<0.0f || font_scale>10.0f) {
             conffile.invalidValueException(entry);
         }
 
-        font_size         = glm::clamp((int)(font_size * font_scale), 1, 100);
-        user_font_size    = glm::clamp((int)(user_font_size * font_scale), 1, 100);
-        dirname_font_size = glm::clamp((int)(dirname_font_size * font_scale), 1, 100);
-    }    
+        setScaledFontSizes();
+    }
 
     if((entry = gource_settings->getEntry("hash-seed")) != 0) {
 
@@ -1207,6 +1228,19 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("file-idle-time-at-end")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify file-idle-time-at-end (seconds)");
+
+        std::string file_idle_at_end_str = entry->getString();
+
+        file_idle_time_at_end = (float) atoi(file_idle_at_end_str.c_str());
+
+        if(file_idle_time_at_end<0.0f || (file_idle_time_at_end == 0.0f && file_idle_at_end_str[0] != '0') ) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
     if((entry = gource_settings->getEntry("user-idle-time")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify user-idle-time (seconds)");
@@ -1340,6 +1374,10 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         stop_on_idle = true;
     }
 
+    if(gource_settings->getBool("fixed-user-size")) {
+        fixed_user_size = true;
+    }
+
     if((entry = gource_settings->getEntry("max-files")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify max-files (number)");
