@@ -22,8 +22,10 @@
 
 PositionSlider::PositionSlider(float percent) {
     this->percent = percent;
+}
 
-    font = fontmanager.grab(gGourceSettings.font_file, 16);
+void PositionSlider::init() {
+    font = fontmanager.grab(gGourceSettings.font_file, 16 * gGourceSettings.font_scale);
     font.dropShadow(true);
 
     slidercol = vec3(1.0, 1.0, 1.0);
@@ -34,6 +36,8 @@ PositionSlider::PositionSlider(float percent) {
     fade_time = 1.0;
     alpha = 0.0;
 
+    capwidth = 0.0f;
+
     resize();
 }
 
@@ -153,7 +157,8 @@ void PositionSlider::draw(float dt) {
     glColor4f(1.0, 1.0, 1.0, 1.0);
 
     if(caption.size() && mouseover >= 0.0) {
-        font.draw(std::min((double)display.width - capwidth - 1.0, std::max(1.0, mouseover - (capwidth/2.0))), bounds.min.y - 25.0, caption);
+        int height_offset = 25 * gGourceSettings.font_scale;
+        font.draw(std::min((double)display.width - capwidth - 1.0, std::max(1.0, mouseover - (capwidth/2.0))), bounds.min.y - height_offset, caption);
     }
 
 }
